/*****************************************
 *                                       *
 *  JBoss Portal: The OpenSource Portal  *
 *                                       *
 *   Distributable under LGPL license.   *
 *   See terms of license at gnu.org.    *
 *                                       *
 *****************************************/
package org.jboss.wiki.fileaccess;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.activation.MimetypesFileTypeMap;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.wiki.FileDataSource;
import org.jboss.wiki.WikiPlugin;

// import org.jboss.forge.common.content.ContentManager;
// import org.jboss.forge.common.ForgeHelper;

/**
 * A filter that enables to download files form the jspwiki structure +
 * redirects to pages by typing http://domain/wiki/Page
 * 
 * @author tomek
 */

public class WikiFileAccessFilter implements Filter {
	/**
	 * <code>BUF_SIZE</code> - size of a buffer, which is used while reading a
	 * file and sending it to the client.
	 */
	
	public  static final String specialPage = "!WikiButtons";
	private final int BUF_SIZE = 32768;

	private String attachementsPath;

	private String wikiHome;
	
	private String[] replaces;

	/*
	 * private ForgeHelper forgeHelper; private ContentManager contentManager;
	 */
	private MimetypesFileTypeMap mimeTypes;

	public void init(FilterConfig conf) {
		// load mime types map

		mimeTypes = new MimetypesFileTypeMap(WikiFileAccessFilter.class
				.getResourceAsStream("mime.types"));

		// load path to attachements from filedatasource.properties

		Properties props = new Properties();

		try {
			props.load(FileDataSource.class
					.getResourceAsStream("filedatasource.properties"));

			// get the path to attachment property
			attachementsPath = props.getProperty("pathToAttachements");

			props.load(WikiPlugin.class
					.getResourceAsStream("wikiPlugins.properties"));

			// get the host
			wikiHome = props.getProperty("wikiHome");
			
		} catch (IOException e) {
			System.err.println(e);
		}

		System.out.println("[WikiFileAccess]Loaded attachements path: "
				+ attachementsPath);
	}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException {

		if (request instanceof HttpServletRequest) {
			HttpServletRequest httpRequest = (HttpServletRequest) request;
			HttpServletResponse httpResponse = (HttpServletResponse) response;

			String requestURI = httpRequest.getRequestURI();

			// Parse URI to get filename and page name
			String[] tokens = requestURI.split("[/]");

			//System.out.println("length " + tokens.length);

			if (tokens[tokens.length - 2].equals(specialPage)) {
				// show apropriate buttons
				InputStream imageIS = WikiFileAccessFilter.class.getResourceAsStream("/"+tokens[tokens.length - 1]);
				
				ServletOutputStream os = response.getOutputStream();

				response.setContentType("image/png");

				// Transferring the bytes
				try {
					byte[] buffer = new byte[BUF_SIZE];
					int read;
					while ((read = imageIS.read(buffer)) != -1)
						os.write(buffer, 0, read);
				} catch (Exception e2) {
					// Nothing that we can really do. Just send an incomplete
					// file.
				} finally {
					os.close();
					imageIS.close();
				}
				
			}
			else if (tokens.length == 3) {
				String hostURL = ((request.isSecure()) ? "https" : "http") + "://"+request.getServerName()+((request.getServerPort() == 80) ? "" : ":"+ request.getServerPort());
				
				httpResponse.sendRedirect(hostURL+wikiHome + "&page="
						+ tokens[tokens.length - 1]);
			} else {
				// get fileName of attachement and page which is connected to
				String fileName = tokens[tokens.length - 1];
				String pageName = tokens[tokens.length - 2];

				//System.out.println("[WikiFileAccess]Getting file: " + fileName
				//		+ " of page: " + pageName);

				Properties props = new Properties();

				try {
					props.load(new FileInputStream(new File(attachementsPath
							+ "/" + pageName + "-att/" + fileName
							+ "-dir/attachment.properties")));
				} catch (IOException e) {
					System.err.println(e);
				}

				// looking for the newest version
				int version = 1;

				while (props.getProperty((version + 1) + ".author") != null) {
					version++;
				}

				tokens = fileName.split("\\.");

				File file = new File(
						attachementsPath
								+ "/"
								+ pageName
								+ "-att/"
								+ fileName
								+ "-dir/"
								+ version
								+ "."
								+ ((fileName.indexOf(".") != -1) ? tokens[tokens.length - 1]
										: "bin"));

				FileInputStream fileIS = new FileInputStream(file);

				ServletOutputStream os = response.getOutputStream();

				response.setContentType(mimeTypes.getContentType(file));
				response.setContentLength((int) file.length());

				// Transferring the bytes
				try {
					byte[] buffer = new byte[BUF_SIZE];
					int read;
					while ((read = fileIS.read(buffer)) != -1)
						os.write(buffer, 0, read);
				} catch (Exception e2) {
					// Nothing that we can really do. Just send an incomplete
					// file.
				} finally {
					os.close();
					fileIS.close();
				}
			}

		} else {
			throw new IOException(
					"This servlet should be called as HTTPServletRequest");
		}
	}

	public void destroy() {

	}
}
